﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace NeuralNumberNetwork {
    class Neuron {

        public Neuron(String Name) {
            this.Name = Name;

        }

        public Neuron() {
            this.Name = NeuralHelper.getRandomName(20);
        }

        public Neuron(double Activation, bool isBase) {
            this.Activation = Activation;
            this.Name = NeuralHelper.getRandomName(20);
            this._IsBaseNeuron = isBase;
        }
        

        /// <summary>
        /// Name of the Neuron
        /// </summary>
        public String Name { get; }

        /// <summary>
        /// Is the degree of Activation of the Neuron
        /// </summary>
        private double _Activation = -1 ;
        public double Activation {
            get {
                return _Activation;
            }
            set {
                if (value > 1 || value < 0) {
                    throw new ArgumentException("Neuron Activation has to be between 0 and 1.");
                }
                _Activation = value;
            }
        }

        /// <summary>
        /// The Bias of the Neuron
        /// </summary>
        public double Bias { get; set; }


        /// <summary>
        /// Determines if Neuron is in the starter layer of the network
        /// </summary>
        private readonly bool _IsBaseNeuron = false;
        public bool IsBaseNeuron { get { return _IsBaseNeuron; }  }


        /// <summary>
        /// List of all the Predeccessor Neurons connected to the Neuron
        /// </summary>
        private List<Neuron> _PredeccessorNeurons = new List<Neuron>();
        public List<Neuron> PredeccessorNeurons { get { return _PredeccessorNeurons; } }


        /// <summary>
        /// List of all the Weights of the Predeccessor Neurons connected to this Neuron
        /// </summary>
        private List<double> _PredeccessorWeights = new List<double>();
        public List<double> PredeccessorWeights { get { return _PredeccessorWeights; } }

        /// <summary>
        /// Calculation of the Activation using recursion
        /// </summary>
        /// <returns></returns>
        public double EvaluateActivation() {
            //Evalutation only on inner Neurons need when they have not been evaluated before
            if (!this.IsBaseNeuron && this.Activation==-1) {
                double weightedSum = 0;
                for (int i = 0; i < this.PredeccessorNeurons.Count; i++) {
                    weightedSum += this.PredeccessorNeurons[i].EvaluateActivation() * this.PredeccessorWeights[i];
                }

                //Subtract the Bias
                weightedSum = weightedSum - Bias;

                //Calculate Sigmoid function (e^x)/(1+e^x), optimization possible 
                this.Activation = NeuralHelper.SigmoidFunction(weightedSum);            
            }
            return this.Activation;
        }


        public void randomizeWeights() {
            Random rand = new Random();
            //Order of weight to Neuron is mixed but does not matter since it is random anyways
            for (int i = 0; i < this.PredeccessorNeurons.Count; i++) {
                this.PredeccessorWeights.Add(rand.NextDouble());
            }
        }

        public override string ToString() {
            return this.Name + " :" + this.Activation;  
        }
    }
}
